# -*- coding: utf-8 -*-

from django.core.management.base import AppCommand, CommandError
from django.conf import settings
from legume.models import *
import datetime

import os, sys

class Command(AppCommand):
    help = "Check the state of action queue"

    def handle(self, *apps, **options):
        nb_failed = Task.objects.filter(state = TASK_STATE_ABORTED).count()
        if nb_failed:
            print "Critical : %d tasks are aborted (definitely failed)" % nb_failed
            sys.exit(2)

        nb_failed = Task.objects.filter(state = TASK_STATE_FAILED).count()
        if nb_failed:
            print "Warning : %d tasks are (temporarily) failed" % nb_failed
            sys.exit(1)

        pending = Task.objects.pending()
        tasks = []
        when = datetime.datetime.now() - datetime.timedelta(hours = 1)
        for task in pending:
            if not task.available():
                continue
            tasktime = task.run_on or task.created_on
            if tasktime < when:
                tasks.append(task)
        nb_tasks = len(tasks)
        if nb_tasks:
            print "Warning : %d tasks are scheduled since more than one hour" % nb_tasks
            sys.exit(1)

        print "All good"
        sys.exit(0)
