# -*- coding: utf-8 -*-

from django.core.management.base import AppCommand, CommandError
from django.conf import settings
from django.db.utils import DatabaseError
from django.db import close_connection

from legume.models import *
from legume.task import run_task, schedule_periodic_tasks
from legume.loggers import logger

import time
import os, sys

class Command(AppCommand):
    help = "Process legume actions without daemonizing"

    def handle(self, *apps, **options):
        self.run_imports()
        self.reset_running_tasks()
        self.perform_loop()

    def run_imports(self):
        """
        Import the "tasks"
        """
        logger.info("Starting at PID %d..." % os.getpid())
        for app in settings.INSTALLED_APPS:
            try:
                module = __import__(app + '.tasks', {}, {}, ['TASKS'])
            except ImportError:
                pass
        logger.debug("Actions loaded")

    def reset_running_tasks(self):
        """
        On reboot reset runnning tasks 
        """
        Task.objects.filter(state=TASK_STATE_RUNNING).update(state=TASK_STATE_SCHEDULED)

    def perform_loop(self):
        """
        Perform the endless loop
        """
        delay = getattr(settings, 'LEGUME_DELAY', 1)

        while True:
            try:
                self.perform_step()
            except KeyboardInterrupt:
                raise
            except SystemExit:
                raise
            except DatabaseError:
                logger.warning("Got a database error, trying to reconnect...")
                close_connection()
            except Exception, e:
                logger.exception("Got unhandled exception in perform_step")
            logger.debug("Sleeping for %.2f second(s)..." % delay)
            time.sleep(delay)

    def perform_step(self):
        """
        Perform one step
        """
        schedule_periodic_tasks()
        task = Task.objects.pop()
        if task:
            logger.debug("Got task %s" % task)
            run_task(task)
        else:
            logger.debug("No task, waiting")

