# -*- coding: utf-8 -*-

from django.core.management.base import AppCommand, CommandError
from django.conf import settings
import daemon, atexit
from legume.loggers import logger

import os, sys, errno, time, signal

def pid_exists(pid):
    """Check whether pid exists in the current process table.
    UNIX only.
    """
    try:
        os.kill(pid, 0)
    except OSError as err:
        if err.errno == errno.ESRCH:
            # ESRCH == No such process
            return False
        elif err.errno == errno.EPERM:
            # EPERM clearly means there's a process to deny access to
            return True
        else:
            # According to "man 2 kill" possible error values are
            # (EINVAL, EPERM, ESRCH)
            raise
    else:
        return True

class Command(AppCommand):
    help = "Start/stop the legume daemon"

    def handle(self, *args, **options):
        """
        Handle the command
        """
        arg = len(args) == 1 and args[0] or None
        if arg == "start":
            self.start()
        elif arg == "stop":
            self.stop()
        elif arg == "restart":
            self.stop()
            time.sleep(1)
            self.start()
        else:
            print u"Must specify one of : start, stop, restart or graceful"
            sys.exit(1)

    def start(self):
        """
        Start the whole machinery
        """
        # Check if DEBUG mode is ON
        if settings.DEBUG:
            print u"Turn off django DEBUG mode or use legume_process_actions command"
            sys.exit(1)
        # Check if pidfile already exists
        pidfile = self.pidfile()
        if os.path.exists(pidfile):
            if pid_exists(self.getpid()):
                print u"Process already running"
                sys.exit(1)

        with daemon.DaemonContext(working_directory = os.getcwd()):
            self.write_pid()
            os.execl(sys.argv[0], sys.argv[0], 'legume_process_actions')

    def pidfile(self):
        """
        Get the PID file
        """
        return os.path.join(settings.LEGUME_PID_DIR, 'legume.pid')

    def getpid(self):
        """
        Get the numeric PID of running instance
        """
        return int(open(self.pidfile()).read().strip())

    def write_pid(self):
        """
        Write the PID file
        """
        pidfile = self.pidfile()
        open(pidfile, 'w', 0664).write(str(os.getpid()))
        atexit.register(lambda: os.remove(pidfile))

    def stop(self):
        """
        Stop the daemon
        """
        try:
            os.kill(self.getpid(), signal.SIGTERM)
        except Exception, e:
            print "error", e

